/*
 * imagerect.c: Mapping of where user clicks returning different things
 * 
 * Rob McCool
 * 
 */


#include <stdio.h>

#define RECT_CONF "/usr/local/etc/httpd/conf/imagerect.conf"
#define LF 10
#define CR 13

void getword(char *word, char *line, char stop);
char x2c(char *what);
void unescape_url(char *url);
void plustospace(char *str);
int rind(char *s, char c);
int getline(char *s,int n, FILE *f);
void send_fd(FILE *f, FILE *fd);

void getdoc(char *path, char *name) {
    register int x=rind(path,'/');

    strcpy(name, (x == -1 ? path : &path[x+1]));
    return;
}

#define USAGE 0
#define CONFIG 1
#define BADFILE 2
#define NOTIMAGERECT 3

void print_error(int err,char *reason) {
    printf("Content-type: text/html%c%c",LF,LF);
    switch(err) {
      case USAGE:
        printf("<TITLE>ImageRect Usage Error</TITLE>%c",LF);
        printf("<H1>ImageRect Usage Error</H1>%c",LF);
        printf("<BODY>This script has been incorrectly accessed. The %c",LF);
	printf("probable cause is that your client doesn't support the%c",LF);
	printf("HTML ISMAP directive yet. <P>%c",LF);
        printf("If you are developing a client, the usage is: <P>%c",LF);
        printf("<CODE>/htbin/%s?x,y</CODE>%c",reason,LF);
        printf("Please see the documentation for setup details.</BODY>%c",LF);
        break;
      case CONFIG:
        printf("<TITLE>ImageRect Configuration Error</TITLE>%c",LF);
        printf("<H1>ImageRect Configuration Error</H1>%c",LF);
        printf("<BODY>This script is not set up properly on this%c",LF);
        printf("server. The configuration file %s is unreadable.%c",
               reason,LF);
        break;
      case BADFILE:
        printf("<TITLE>ImageRect Configuration Error</TITLE>%c",LF);
        printf("<H1>ImageRect Configuration Error</H1>%c",LF);
        printf("<BODY>The file %s could not be opened and sent.%c",reason,LF);
        break;
      case NOTIMAGERECT:
        printf("<TITLE>ImageRect Configuration Error</TITLE>%c",LF);
        printf("<H1>ImageRect Configuration Error</H1>%c",LF);
        printf("The file %s is not in the imagerect configuration file.<P>%c",
               reason,LF);
        break;
    }
    exit(1);
}

main(int argc, char *argv[]) {
    FILE *f,*rt;
    char name[256];
    char l[256];
    char w[256];
    char ct[256];
    char defname[256];
    char deftype[256];
    int x,y,xmin,xmax,ymin,ymax;

    getdoc(argv[0],name);
    if(argc != 2)
        print_error(USAGE,name);

    if(!(f=fopen(RECT_CONF,"r")))
        print_error(CONFIG,RECT_CONF);

    while(!(getline(l,256,f))) {
        getword(w,l,' ');
        if(!(strcmp(name,w))) {
            getword(defname,l,' ');
            getword(deftype,l,' ');

            getword(w,argv[1],',');
            x=atoi(w);
            getword(w,argv[1],' ');
            y=atoi(w);
            while(!getline(l,256,f)) {
                if(l[0] == '}') break;
                sscanf(l,"%d%d%d%d%s%s",&xmin,&xmax,&ymin,&ymax,w,ct);
                if((x >= xmin) && (x <= xmax) && (y >= ymin) && (y <= ymax)) {
                    if(!(rt=fopen(w,"r")))
                        print_error(BADFILE,w);
                    fprintf(stdout,"Content-type: %s%c%c",ct,LF,LF);
                    send_fd(rt,stdout);
                    exit(0);
                }
            }
            if(!(rt=fopen(defname,"r")))
                print_error(BADFILE,defname);
            fprintf(stdout,"Content-type: %s%c%c",deftype,LF,LF);
            send_fd(rt,stdout);
            exit(0);
        }
    }
    print_error(NOTIMAGERECT,name);
}
