/*
 * http_request.c: functions to get and process requests
 * 
 * Rob McCool 3/21/93
 * 
 * Include code by Charles Henrich
 */


#include "httpd.h"

int assbackwards;

void send_fd(FILE *f, FILE *fd, char *args);

/*
 * Thanks to Charles Henrich
 */
void process_include(FILE *f, FILE *fd, char *incstring, char *args)
{
    FILE *ifp;
    char srcfile[HUGE_STRING_LEN];
    char command[HUGE_STRING_LEN];
    char errstr[MAX_STRING_LEN];
    char cmd[10];
    int num_includes;
    
    if(sscanf(incstring,"%s \"%[^\"]", cmd, srcfile) != 2) {
        sprintf(errstr,"the include string %s was invalid",incstring);
        die(INCLUDE_ERROR,errstr,fd);
    }                
    
    if(strncasecmp(cmd,"srv",3) != 0) {
        fprintf(fd, "%s\r\n",cmd);
        return;
    }
    
    if(srcfile[0] != '|') {
        if(num_includes > MAXINCLUDES) {
            die(INCLUDE_ERROR,
                "the maximum number of includes has been exceeded",fd);
        }
        
        num_includes++;
        
        translate_name(srcfile);
        ifp=fopen(srcfile,"r");
        
        if(ifp != NULL) {
            send_fd(ifp, fd, "");
            fclose(ifp);
        }
        else {
            sprintf(errstr,"the include file %s was invalid",srcfile);
            die(INCLUDE_ERROR,errstr,fd);
        }
    }
    else {
        if(strncasecmp(cmd,"srvurl",6) == 0) {
            sprintf(command,"%s '%s'",&srcfile[1],args);
        }
        else {
            strcpy(command,&srcfile[1]);
        }
            
        ifp=popen(command,"r");
            
        if(ifp != NULL) {
            send_fd(ifp, fd, args);
            pclose(ifp);
        }
        else {
            sprintf(errstr,"the command %s is invalid",&srcfile[1]);
            die(INCLUDE_ERROR,errstr,fd);
        }
    }
}


void send_fd(FILE *f, FILE *fd, char *args)
{
    int num_chars=0;
    char c;
    struct stat finfo;

    if(!assbackwards) {
/*        fstat(f,&finfo); */
/*        set_content_length(finfo.st_size); */
/*        set_last_modified(finfo.st_mtime); */
        send_mime_headers(fd);
    }
    if(is_content_type("text/html")) {
        char *find="<inc";
        char incstring[MAX_STRING_LEN];
        int x,p;

        p=0;
        while(1) {
            c = fgetc(f);
            if(feof(f)) return;
            if(c == find[p]) {
                if((++p) == 4) {
                    x=0;
                    c=fgetc(f); /* get the space instead of the c */
                    while(c != '>') {
                        incstring[x++] = c;
                        c = fgetc(f);
                        if(feof(f)) {
                            incstring[x] = '\0';
                            fputs("<inc",fd);
                            fputs(incstring,fd);
                            return;
                        }
                    }
                    incstring[x] = '\0';
                    process_include(f,fd,incstring,args);
                    p=0;
                }
            }
            else {
                if(p) {
                    for(x=0;x<p;x++)
                        fputc(find[x],fd);
                    p=0;
                }
                fputc(c,fd);
            }
        }
    } else {
        while (1) {
            c = fgetc(f);
            if(feof(f))
                return;
            fputc(c,fd);
        }
    }
}

void send_file(char *file, FILE *fd, char *args) {
    FILE *f;

    if(!(f=fopen(file,"r"))) {
        unmunge_name(file);
        die(FORBIDDEN,file,fd); /* we've already established that it exists */
    }
    set_content_type(file);
    send_fd(f,fd,args);
}

void exec_script(char *path, char *args, FILE *fd) {
    char command[HUGE_STRING_LEN*2];
    char tfile[L_tmpnam];
    FILE *tfp;

    tmpnam(tfile);
    escape_shell_cmd(args);
    escape_shell_cmd(path);
    sprintf(command,"%s %s > %s 2>&1",path,args,tfile);
    system(command);
    tfp = fopen(tfile,"r");
    scan_content_type(tfp);
    send_fd(tfp,fd,args);
    fclose(tfp);
    if(unlink(tfile) == -1) {
        char err[MAX_STRING_LEN];
        sprintf(err,"could not remove temp file %s",tfile);
        log_error(err);
    }
}

/* GET */
void send_node(char *file, char *args, FILE *fd)
{
    struct stat finfo;

    if(stat(file,&finfo) == -1) {
        unmunge_name(file);
        if(errno == ENOENT) die(NOT_FOUND,file,fd);
        die(FORBIDDEN,file,fd);
    }

    if(!strncmp(bindir,file,strlen(bindir) - 1)) {
        exec_script(file,args,fd);
        return;
    }

    if(!no_indexes) {
        if(S_ISDIR(finfo.st_mode)) {
            char ifile[MAX_STRING_LEN];
            
            make_full_path(file,DEFAULT_INDEX,ifile);
            if(stat(ifile,&finfo) == -1)
                /* index file not found; construct */
                index_directory(file,fd);
            else send_file(ifile,fd,args);
            return;
        }
    }
    if(S_ISREG(finfo.st_mode))
        send_file(file,fd,args);
    else {
        unmunge_name(file);
        die(FORBIDDEN,file,fd); /* device driver or pipe, no permission */
    }
}

void process_request(FILE *in, FILE *out) {
    char w[HUGE_STRING_LEN];
    char l[HUGE_STRING_LEN];
    char url[HUGE_STRING_LEN];
    char args[HUGE_STRING_LEN];

    who_called = get_peer_hostname(fileno(in));
    getline(l,1024,in,TIMEOUT);

    log_transaction(l);
    getword(w,l,' ');
    if(!strcmp(w,"GET")) {
        getword(args,l,' ');
        getword(url,args,'?');
        unescape_url(url);
        plustospace(args);
        getword(w,l,'\0');
        if(w[0] != '\0') {
            assbackwards = 0;
            get_mime_headers(in);
        }
        else assbackwards = 1;
        translate_name(url);
        send_node(url,args,out);
    }
    else {
        char errstr[MAX_STRING_LEN];
        sprintf(errstr,"unknown method \"%s\"",w);
        die(BAD_REQUEST,errstr,out);
    }
}
