/*
 * httpd.h: header for simple (ha! not anymore) http daemon
 */


/* Define if your system calls are BSD-style */
#define BSD

/* ----------------------------- config dir ------------------------------ */

/* Define this to be the default server home dir. Anything later in this
 * file with a relative pathname will have this added.
 */
#define HTTPD_ROOT "/usr/local/etc/httpd"

/* -------------- Port number for server running standalone --------------- */

#define DEFAULT_PORT 80

/* --------- Default user name and group name running standalone ---------- */
/* --- These may be specified as numbers by placing a # before a number --- */

#define DEFAULT_USER "nobody"
#define DEFAULT_GROUP "#-1"

/* The name of the log files */
#define DEFAULT_XFERLOG "logs/access_log"
#define DEFAULT_ERRORLOG "logs/error_log"

/* The default string lengths */
#define MAX_STRING_LEN 256
#define HUGE_STRING_LEN 8192

/* Define this to be what your HTML directory content files are called */
#define DEFAULT_INDEX "index.html"

/* The name of the server config file */
#define SERVER_CONFIG_FILE "conf/httpd.conf"

/* The name of the MIME types file */
#define MIME_TYPES_FILE "conf/mime.types"

/* The default binary directory name */
#define DEFAULT_BIN "htbin"

/* The timeout for waiting for messages */
#define TIMEOUT 60

/* Root of server */
#define DOCUMENT_LOCATION "/usr/local/etc/http/htdocs"

/* Max. number of aliases */
#define MAX_ALIASES 20

/* Max. number of include files */
#define MAXINCLUDES 20

/* Internal name for non-existent dir. */
#define DENY_NAME "/DENY!DENY!DENY!"

/* Default administrator's address */
#define DEFAULT_ADMIN ""

/*
 * The particular directory style your system supports. If you have dirent.h
 * in /usr/include (POSIX) or /usr/include/sys (SYSV), #include 
 * that file and define DIR_TYPE to be dirent. Otherwise, if you have 
 * /usr/include/sys/dir.h, define DIR_TYPE to be direct and include that
 * file. If you have neither, I'm confused.
 */
#include <dirent.h>
#define DIR_TYPE dirent

/* --------- You shouldn't have to edit anything below this line ---------- */

/* ------------------------------ error types ------------------------------ */

#define SERVER_VERSION "NCSA/1.0a"
#define SERVER_PROTOCOL "HTTP/1.0"
#define SERVER_SUPPORT "httpd@ncsa.uiuc.edu"

#define BAD_REQUEST 400
#define FORBIDDEN 403
#define NOT_FOUND 404
#define SERVER_ERROR 500
#define NOT_IMPLEMENTED 501
#define INCLUDE_ERROR 6991

#include "/usr/include/bsd/signal.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <sys/socket.h>
#include <ctype.h>
#include <netinet/in.h>
#include <netdb.h>
#include <sys/ioctl.h>
#include <arpa/inet.h>  /* for inet_ntoa */
#include <time.h>  /* for ctime */
#include <signal.h>
#include <errno.h>
#include <sys/wait.h>
#include <pwd.h>
#include <grp.h>


#ifdef NEXT_BRAIN_DEATH
#define S_ISDIR(m)      (((m)&(S_IFMT)) == (S_IFDIR))
#define S_ISREG(m)      (((m)&(S_IFMT)) == (S_IFREG))
#endif

#ifdef ultrix
#define ULTRIX_BRAIN_DEATH
#endif

/* Just in case your linefeed isn't the one the other end is expecting. */
#define LF 10
#define CR 13

/* Globals */
extern int standalone;
extern int port;
extern uid_t user_id;
extern gid_t group_id;
extern char server_root[MAX_STRING_LEN];
extern FILE *error_log;
extern char error_fname[MAX_STRING_LEN];
extern FILE *xfer_log;
extern char xfer_fname[MAX_STRING_LEN];
extern char bindir[MAX_STRING_LEN];
extern char index_name[MAX_STRING_LEN];
extern char document_root[MAX_STRING_LEN];
extern int no_indexes;
extern char server_admin[MAX_STRING_LEN];

extern char *who_called;
extern int assbackwards;

/* Function prototypes. */

/* http_config */
extern void read_config();

/* http_alias */
extern void reset_aliases();
extern void dump_aliases();
extern void add_alias(char *f, char *r);
extern void translate_name(char *name);
extern void unmunge_name(char *name);

/* http_request */
extern void send_node(char *name, char *args, FILE *fd);
extern void process_request(FILE *in, FILE *out);
extern void send_file(char *file,FILE *fd, char *args);
extern void process_include(FILE *f, FILE *fd, char *incstring, char *args);

/* http_dir */
extern void index_directory(char *name, FILE *fd);

/* http_log */
extern void log_transaction(char *cmd_line);
extern void log_error(char *err);
extern void die(int type, char *err_string, FILE *fd);
extern void open_logs();

/* http_mime */
extern void get_mime_headers(FILE *fd);
extern void send_mime_headers(FILE *fd);
extern void set_content_type(char *fn);
extern void scan_content_type(FILE *f);
extern void set_content_length(int l);
extern void dump_types();
int is_content_type(char *type);

/* util */
extern void strsubfirst(int start,char *dest, char *src);
extern void make_full_path(char *src1,char *src2,char *dst);
extern int is_directory(char *name);
extern void getparents(char *name);
extern uid_t uname2id(char *name);
extern gid_t gname2id(char *name);
extern int getline(char *s, int n, FILE *f, unsigned int timeout);
extern void getword(char *word, char *line, char stop);
extern char *get_peer_hostname(int fd);
extern char *get_time();
extern char *get_gmt();
extern void unescape_url(char *url);
extern void escape_url(char *url);
extern void escape_shell_cmd(char *cmd);
extern void plustospace(char *str);
#ifdef NEED_STRDUP
extern char *strdup (char *str);
#endif
int ind(char *s, char c);
int rind(char *s, char c);
