/*
 * str.c: string utility things
 * 
 * 3/21/93 Rob McCool
 * 
 */


#include "httpd.h"

char *get_time() {
    time_t t;
    char *time_string;

    t=time(NULL);
    time_string = ctime(&t);
    time_string[strlen(time_string) - 1] = '\0';
    return (time_string);
}

char *get_gmt() {
    time_t sec;
    char *time_string;
    struct tm *t;

    sec=time(NULL);
    t = gmtime(&sec);
    time_string = asctime(t);
    time_string[strlen(time_string) - 1] = '\0';
    return (time_string);
}

void strsubfirst(int start,char *dest, char *src)
{
    char tmp[MAX_STRING_LEN];

    strcpy(tmp,&dest[start]);
    strcpy(dest,src);
    strcpy(&dest[strlen(src)],tmp);
}

/*
 * Parse .. so we don't compromise security
 */
void getparents(char *name)
{
    int l=0,w=0;
    const char *lookfor="..";

    while(name[l]!='\0') {
        if(name[l]!=lookfor[w]) (w>0 ? (l-=(w-1),w=0) : l++);
        else {
            if(lookfor[++w]=='\0') {
                if((name[l+1]=='\0') || (name[l+1]=='/')) {
                    register int m=l+1,n;

                    l=l-3;
                    if(l>=0) {
                        while((l!=0) && (name[l]!='/')) --l;
                    }
                    else l=0;
                    n=l;
                    while(name[n]=name[m]) (++n,++m);
                    w=0;
                }
                else w=0;
            }
            else ++l;
        }
    }
}

void getline_timed_out() {
    char errstr[MAX_STRING_LEN];

    sprintf(errstr,"timed out waiting for %s",who_called);
    log_error(errstr);
    fclose(stdin);
    fclose(stdout);
    exit(0);
}

int getline(char *s, int n, FILE *f, unsigned int timeout) {
    register int i=0;

    signal(SIGALRM,getline_timed_out);
    alarm(timeout);
    while(1) {
        s[i] = (char)fgetc(f);

        if(s[i] == CR)
            s[i] = fgetc(f);

        if((s[i] == EOF) || (s[i] == LF) || (i == (n-1))) {
            alarm(0);
            signal(SIGALRM,SIG_IGN);
            s[i] = '\0';
            return (feof(f) ? 1 : 0);
        }
        ++i;
    }
}

void getword(char *word, char *line, char stop) {
    int x = 0,y;

    for(x=0;((line[x]) && (line[x] != stop));x++)
        word[x] = line[x];

    word[x] = '\0';
    if(line[x]) ++x;
    y=0;

    while(line[y++] = line[x++]);
}

void plustospace(char *str) {
    register int x;

    for(x=0;str[x];x++) if(str[x] == '+') str[x] = ' ';
}

char x2c(char *what) {
    register char digit;

    digit = ((what[0] >= 'A') ? ((what[0] & 0xdf) - 'A')+10 : (what[0] - '0'));
    digit *= 16;
    digit += (what[1] >= 'A' ? ((what[1] & 0xdf) - 'A')+10 : (what[1] - '0'));
    return(digit);
}

void unescape_url(char *url) {
    register int x,y;

    for(x=0;url[x];x++) {
        if(url[x] == '%') {
            url[x] = x2c(&url[x+1]);
            y=x+1;
            while(url[y] = url[y+2]) y++;
        }
    }
}

void c2x(char what, char *where) {
    register int x;
    char foom[MAX_STRING_LEN];

    for(x=0;where[x];x++)
        foom[x] = where[x+1];

    sprintf(where,"%%%2x%s",what,foom);
}

void escape_url(char *url) {
    register int x,y;
    register char digit;

    for(x=0;url[x];x++) {
        if(ind("% ?+",url[x]) != -1) {
            c2x(url[x],&url[x]);
            x+=2;
        }
    }
}

void escape_shell_cmd(char *cmd) {
    register int x,y,l;

    l=strlen(cmd);
    for(x=0;cmd[x];x++) {
        if(ind("&*<>\\",cmd[x]) != -1){
            for(y=l+1;y>x;y--)
                cmd[y] = cmd[y-1];
            l++; /* length has been increased */
            cmd[x] = '\\';
            x++; /* skip the character */
        }
    }
}

void make_full_path(char *src1,char *src2,char *dst) {
    while(*dst++ = *src1++);
    if(*(dst-2) != '/')
        *(dst-1)='/';
    else --dst;
    while(*dst++ = *src2++);
}

int is_directory(char *path) {
    struct stat finfo;

    if(stat(path,&finfo) == -1)
        return 0; /* in error condition, just return no */

    return(S_ISDIR(finfo.st_mode));
}

#ifdef NEED_STRDUP
char *strdup (char *str)
{
  char *dup;

  dup = (char *)malloc (strlen (str) + 1);
  dup = strcpy (dup, str);

  return dup;
}
#endif

int ind(char *s, char c) {
    register int x;

    for(x=0;s[x];x++)
        if(s[x] == c) return x;

    return -1;
}

int rind(char *s, char c) {
    register int x;

    for(x=strlen(s)-1;x != -1;x--)
        if(s[x] == c) return x;

    return -1;
}

uid_t uname2id(char *name) {
    struct passwd *ent;

    if(name[0] == '#') 
        return(atoi(&name[1]));

    if(!(ent = getpwnam(name))) {
        fprintf(stderr,"httpd: bad user name %s\n",name);
        exit(1);
    }
    else return(ent->pw_uid);
}

gid_t gname2id(char *name) {
    struct group *ent;

    if(name[0] == '#') 
        return(atoi(&name[1]));

    if(!(ent = getgrnam(name))) {
        fprintf(stderr,"httpd: bad group name %s\n",name);
        exit(1);
    }
    else return(ent->gr_gid);
}

char *get_peer_hostname(int fd) {
    struct sockaddr addr;
    int len;
    char *who_called;

    /*
     * Get a sockaddr structure filled in for our peer.
     */
    len = sizeof(struct sockaddr);
    
    if (!(getpeername(fd, &addr, &len))) {
        struct in_addr *iaddr;
        struct hostent *hptr;
        
        iaddr = &(((struct sockaddr_in *)&addr)->sin_addr);
        hptr = gethostbyaddr((char *)iaddr,
                             sizeof(struct in_addr), AF_INET);
        if (hptr != NULL) {
            who_called = (char *)malloc(strlen(hptr->h_name) + 1);
            strcpy(who_called, hptr->h_name);
        }
        else {
            char *iname;
            
            iname = inet_ntoa(*iaddr);
            if (iname != NULL) {
                who_called = (char *)malloc(strlen(iname) + 1);
                strcpy(who_called, iname);
            }
            else {
                who_called = (char *)malloc(strlen("UNKNOWN_CALLER") + 1);
                strcpy(who_called, "UNKNOWN_CALLER");
            }
        }
    }
    else {
        who_called = (char *)malloc(strlen("UNKNOWN_CALLER") + 1);
        strcpy(who_called, "UNKNOWN_CALLER");
    }
    return(who_called);
}
